<?php

namespace Drupal\rich_image\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;

/**
 * Plugin implementation of the 'rich_image' formatter.
 *
 * @FieldFormatter(
 *   id = "rich_image",
 *   label = @Translation("Rich Image"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class RichImageFormatter extends ImageFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'attribution_render' => TRUE,
      'caption_title_render' => TRUE,
      'caption_render' => TRUE,
      'link_render' => TRUE,
      'link_target' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $field_settings = $this->getFieldSettings();
    $field_name = $this->fieldDefinition->getName();

    if (!empty($field_settings['rich_image']['attribution_field'])) {
      $element['attribution_render'] = [
        '#type' => 'checkbox',
        '#title' => t('Render <em>Attribution</em> field'),
        '#default_value' => $this->getSetting('attribution_render'),
        '#weight' => 21,
      ];
    }
    if (!empty($field_settings['rich_image']['caption_title_field'])) {
      $element['caption_title_render'] = [
        '#type' => 'checkbox',
        '#title' => t('Render <em>Caption Title</em> field'),
        '#default_value' => $this->getSetting('caption_title_render'),
        '#weight' => 22,
      ];
    }
    if (!empty($field_settings['rich_image']['caption_field'])) {
      $element['caption_render'] = [
        '#type' => 'checkbox',
        '#title' => t('Render <em>Caption</em> field'),
        '#default_value' => $this->getSetting('caption_render'),
        '#weight' => 23,
      ];
    }
    if (!empty($field_settings['rich_image']['link_field'])) {
      $element['link_render'] = [
        '#type' => 'checkbox',
        '#title' => t('Link image to <em>Link</em> field'),
        '#default_value' => $this->getSetting('link_render'),
        '#weight' => 24,
      ];
      $element['link_target'] = [
        '#type' => 'checkbox',
        '#title' => t('Open link in a new window'),
        '#default_value' => $this->getSetting('link_target'),
        '#return_value' => '_blank',
        '#weight' => 25,
        '#states' => [
          'visible' => [
            ":input[name=\"fields[$field_name][settings_edit_form][settings][link_render]\"]" => ['checked' => TRUE],
          ],
        ],
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $render_settings = $this->getRenderSettings();

    $render = [];
    if (!empty($render_settings['attribution_render'])) {
      $render[] = t('Attribution');
    }
    if (!empty($render_settings['caption_title_render'])) {
      $render[] = t('Caption Title');
    }
    if (!empty($render_settings['caption_render'])) {
      $render[] = t('Caption');
    }
    if (!empty($render)) {
      $render = implode(', ', $render);
      $summary[] = t('Render: @ph1', ['@ph1' => $render]);
    }

    $summary[] = t('Link image:') . ' '
      . (empty($render_settings['link_render'])
        ? t('No')
        : (empty($this->getSetting('link_target'))
          ? t('Open in the same window')
          : t('Open in a new window')));

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    $formatter_settings = ['enigma' => 'anya'];

    foreach ($elements as $delta => $element) {
      // Set a new theme callback function for the image caption formatter.
      $elements[$delta]['#theme'] = 'rich_image_formatter';
      $elements[$delta]['#rich_image_formatter_settings'] = $this->getRenderSettings();
    }

    return $elements;
  }

  /**
   * Override ImageFormatterBase::getEntitiesToView().
   *
   * Because there is too much code that is overridden, I decided to write a
   * patch in RichImageItem::serValue.
   */
  protected function getEntitiesToViewNotUsed(EntityReferenceFieldItemListInterface $items, $langcode) {
    // Add the default image if needed.
    if ($items->isEmpty()) {
      $default_image = $this->getFieldSetting('default_image');
      // If we are dealing with a configurable field, look in both
      // instance-level and field-level settings.
      if (empty($default_image['uuid']) && $this->fieldDefinition instanceof FieldConfigInterface) {
        $default_image = $this->fieldDefinition->getFieldStorageDefinition()->getSetting('default_image');
      }
      if (!empty($default_image['uuid']) && $file = \Drupal::service('entity.repository')->loadEntityByUuid('file', $default_image['uuid'])) {
        // Clone the FieldItemList into a runtime-only object for the formatter,
        // so that the fallback image can be rendered without affecting the
        // field values in the entity being rendered.
        $items = clone $items;
        $items->setValue([
          'target_id' => $file->id(),
          'alt' => $default_image['alt'],
          'title' => $default_image['title'],
          'width' => $default_image['width'],
          'height' => $default_image['height'],
          'attribution' => $default_image['attribution'],
          'entity' => $file,
          '_loaded' => TRUE,
          '_is_default' => TRUE,
        ]);
        $file->_referringItem = $items[0];
      }
    }

    return parent::getEntitiesToView($items, $langcode);
  }

  /**
   * Combines field+formatter settings for a propert to render.
   *
   * @return array
   *   Array of boolean flags whether the properties should be rendered.
   */
  private function getRenderSettings() {
    $static = &drupal_static(__METHOD__, []);

    if (empty($static)) {
      $field_settings = $this->getFieldSettings();
      $settings = $this->getSettings();

      $static['attribution_render'] = !(empty($field_settings['rich_image']['attribution_field'])
        or empty($settings['attribution_render']));
      $static['caption_title_render'] = !(empty($field_settings['rich_image']['caption_title_field'])
        or empty($settings['caption_title_render']));
      $static['caption_render'] = !(empty($field_settings['rich_image']['caption_field'])
        or empty($settings['caption_render']));
      $static['link_render'] = !(empty($field_settings['rich_image']['link_field'])
        or empty($settings['link_render']));
      $static['link_target'] = $this->getSetting('link_target');
    }

    return $static;
  }

}
